from pyspark.sql import SparkSession
from pyspark.sql.functions import col
import matplotlib.pyplot as plt
import numpy as np
from sklearn.cluster import KMeans
from sklearn.preprocessing import StandardScaler
import pandas as pd

# Initialisation Spark
spark = SparkSession.builder.getOrCreate()

# Chargement depuis table Lakehouse
df = spark.sql("SELECT * FROM PythonLakehouse.R_Entries")

# Sélection des colonnes
df_clean = df.select(
    col("ID").alias("ID"),
    col("TpsIndiv").cast("double").alias("TpsIndiv"),
    col("TpsCollab").cast("double").alias("TpsCollab")
).filter(col("ID").isNotNull())

display(df_clean.limit(10))

# Conversion Spark → pandas pour clustering sklearn
df_pandas = df_clean.toPandas()

# Statistiques
df_pandas.describe()

# Clustering
X = df_pandas[['TpsIndiv', 'TpsCollab']].fillna(0)
scaler = StandardScaler()
X_scaled = scaler.fit_transform(X)

kmeans = KMeans(n_clusters=3, random_state=42, n_init=10)
df_pandas['Cluster'] = kmeans.fit_predict(X_scaled)

# Labels automatiques des clusters
centroids = kmeans.cluster_centers_
unscaled_centroids = scaler.inverse_transform(centroids)
intensities = [x + y for x, y in unscaled_centroids]
sorted_indices = np.argsort(intensities)
intensity_labels = ["Temps perdu +", "Temps perdu ++", "Temps perdu +++"]
labels_map = {cluster: intensity_labels[rank] for rank, cluster in enumerate(sorted_indices)}
df_pandas['Clusters'] = df_pandas['Cluster'].map(labels_map)

# Export vers Lakehouse
df_export = df_pandas[['ID', 'TpsIndiv', 'TpsCollab', 'Clusters']]
df_export_spark = spark.createDataFrame(df_export)
df_export_spark.write \
    .option("overwriteSchema", "true") \
    .mode("overwrite") \
    .format("delta") \
    .saveAsTable("R_Clusters")

# Affichage
display(df_export_spark.limit(10))

# Visualisation
for cluster_label in sorted(df_pandas['Clusters'].unique()):
    cluster_data = df_pandas[df_pandas['Clusters'] == cluster_label]
    plt.scatter(cluster_data['TpsIndiv'], cluster_data['TpsCollab'], label=cluster_label)

# Centroïdes
centroids_x, centroids_y = unscaled_centroids[:, 0], unscaled_centroids[:, 1]
plt.scatter(centroids_x, centroids_y, c='black', marker='X', s=200, label='Centroïdes')
plt.axline((0, 0), slope=1, linestyle='--', color='gray', label='y = x')
plt.xlabel("TpsIndiv")
plt.ylabel("TpsCollab")
plt.title("Scatter Clustering")
plt.legend()
plt.show()